/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 2 juillet 2007
   derniere modification le 19/03/2008 */

#ifndef SCEFRAMEBUFFER_H
#define SCEFRAMEBUFFER_H

#include <SCE/utils/SCERectangle.h>

/**
 * \ingroup framebuffer
 */

#ifdef __cplusplus
extern "C"
{
#endif


/* types de buffers */
#define SCE_DEPTH_BUFFER 0
#define SCE_STENCIL_BUFFER 1
#define SCE_COLOR_BUFFER0 2
#define SCE_COLOR_BUFFER1 3
#define SCE_COLOR_BUFFER2 4
#define SCE_COLOR_BUFFER3 5
#define SCE_COLOR_BUFFER4 6
#define SCE_COLOR_BUFFER5 7
#define SCE_COLOR_BUFFER6 8
#define SCE_COLOR_BUFFER7 9
/* nombre de render buffer differents */
#define SCE_NUM_RENDER_BUFFERS 10

#define SCE_COLOR_BUFFER SCE_COLOR_BUFFER0


/* buffer de rendu, soit vers un ID, soit vers une texture */
typedef struct sce_crenderbuffer SCE_CRenderBuffer;
struct sce_crenderbuffer
{
    SCEuint id;        /**< OpenGL render buffer's identifier */
    SCE_CTexture *tex; /**< the render texture */
    int user;          /**< is user the owner of \c tex ? */
};

/* Frame Buffer Object */
typedef struct sce_cframebuffer SCE_CFramebuffer;
struct sce_cframebuffer
{
    SCEuint id;             /* identifiant opengl */
    GLbitfield mask;        /* tampons a mettre a jour */
    /* tampons de ce FBO */
    SCE_CRenderBuffer buffers[SCE_NUM_RENDER_BUFFERS];
    int x, y, w, h; /* TODO: use rectangle ? */
};

/* defini le FBO actif pour les modifications */
void SCE_CBindFramebuffer (SCE_CFramebuffer*);

/* cree un FBO */
SCE_CFramebuffer* SCE_CCreateFramebuffer (void);
/* supprime un FBO */
void SCE_CDeleteFramebuffer (SCE_CFramebuffer*);
void SCE_CDeleteFramebuffer_ (void);

/* ajoute un tampon de rendu vers une texture */
void SCE_CAddRenderTexture (SCE_CFramebuffer*, SCEuint, SCEenum,
                            SCE_CTexture*, int, int);
void SCE_CAddRenderTexture_ (SCEuint, SCEenum, SCE_CTexture*, int, int);

/* ajoute un tampon de rendu vers ... rien :D */
void SCE_CAddRenderBuffer (SCE_CFramebuffer*, SCEuint, int, int, int);
void SCE_CAddRenderBuffer_ (SCEuint, int, int, int);

/* cree une texture et l'ajoute comme render texture */
int SCE_CCreateRenderTexture (SCE_CFramebuffer*, SCEuint,
                              int, int, int, int, int);
int SCE_CCreateRenderTexture_ (SCEuint, int, int, int, int, int);

/* recupere la texture de rendu du render buffer specifie */
SCE_CTexture* SCE_CGetRenderTexture (SCE_CFramebuffer*, SCEuint);
SCE_CTexture* SCE_CGetRenderTexture_ (SCEuint);

/* defini si le buffer specifie du FBO doit etre vide avant le rendu */
void SCE_CClearRenderbuffer (SCE_CFramebuffer*, SCEuint, int);
void SCE_CClearRenderbuffer_ (SCEuint, int);

/* defini le frame buffer actif */
void SCE_CUseFramebuffer (SCE_CFramebuffer*, SCE_SIntRect*);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
