/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19/05/2007
   derniere modification : 05/07/2008 */

#ifndef SCEMATERIAL_H
#define SCEMATERIAL_H

#include <stdarg.h>
#include <SCE/core/SCECMaterial.h>
#include <SCE/core/SCECPointSprite.h>

#include <SCE/interface/SCELight.h>
#include <SCE/interface/SCETexture.h>
#include <SCE/interface/SCEShaders.h>

#ifdef __cplusplus
extern "C"
{
#endif

/* constantes generales */
/* types de maps */
#define SCE_COLOR_MAP 0
#define SCE_NORMAL_MAP 1
#define SCE_HEIGHT_MAP 2
#define SCE_EMISSIVE_MAP 3
#define SCE_GLOSS_MAP 4
#define SCE_CUBE_MAP 5
/* nombre de types de maps */
#define SCE_NUM_MAPS 6

/* identifiants des donnes d'une face d'une cubemap */
#define SCE_TEXID_POSX 0
#define SCE_TEXID_NEGX 1
#define SCE_TEXID_POSY 2
#define SCE_TEXID_NEGY 3
#define SCE_TEXID_POSZ 4
#define SCE_TEXID_NEGZ 5


/* structure d'une texture pour un materiau */
typedef struct sce_smattex SCE_SMatTex;
struct sce_smattex
{
    SCE_STexture *tex; /*! Texture */
    int actived;       /*! True : texture activee */
    int canfree;       /*! True : autorisation de supprimer la texture */
    unsigned int unit; /*! Unite de texture */
};

/* structure d'un shader pour un materiau */
typedef struct sce_smatshd SCE_SMatShd;
struct sce_smatshd
{
    SCE_SShader *shd; /*! Shader */
    int actived;      /*! True : shader active */
    int canfree;      /*! True : autorisation de supprimer le shader */
    int lightindex[SCE_NUM_LIGHT_PARAMS];
};


/**
 * \brief A SCE material
 * 
 * 
 */
typedef struct sce_smaterial SCE_SMaterial;
struct sce_smaterial
{
    SCE_CMaterial *mat;  /**< Core material */
    SCE_SList *maps;     /**< Material's extures */
    SCE_SMatShd shader;  /**< Material's shader */
    SCE_CPointSprite ps; /**< Point sprite's model */
    int use_ps;          /**< Are point sprites used ? */
    int use_sl;          /**< Is shader lighting used ? */
};


/* cree un materiau */
SCE_SMaterial* SCE_Material_Create (void);
/* supprime un materiau */
void SCE_Material_Delete (SCE_SMaterial*);

/* retourne le materiau coeur d'un materiau */
SCE_CMaterial* SCE_Material_GetCMaterial (SCE_SMaterial*);

/* defini une couleur d'un materiau */
void SCE_Material_SetColor(SCE_SMaterial*, SCEenum, float, float, float, float);
void SCE_Material_SetColorv (SCE_SMaterial*, SCEenum, float*);
/* recupere la couleur d'un materiau */
float* SCE_Material_GetColor (SCE_SMaterial*, SCEenum);
void SCE_Material_GetColorv (SCE_SMaterial*, SCEenum, float*);

/* active/desactive les point sprites */
void SCE_Material_ActivatePointSprite (SCE_SMaterial*, int);
void SCE_Material_EnablePointSprite (SCE_SMaterial*);
void SCE_Material_DisablePointSprite (SCE_SMaterial*);
/* defini un modele de point sprite */
void SCE_Material_SetPointSpriteModel (SCE_SMaterial*, SCE_CPointSprite*);
/* renvoie le modele de point sprite */
SCE_CPointSprite* SCE_Material_GetPointSpriteModel (SCE_SMaterial*);

/* defini le blending */
void SCE_Material_ActivateBlending (SCE_SMaterial*, int);
void SCE_Material_EnableBlending (SCE_SMaterial*);
void SCE_Material_DisableBlending (SCE_SMaterial*);
void SCE_Material_SetBlending (SCE_SMaterial*, SCEenum, SCEenum);

/* defini si le materiau supporte la lumiere de type SCE par shaders */
void SCE_Material_ActivateShaderLighting (SCE_SMaterial*, int);
void SCE_Material_EnableShaderLighting (SCE_SMaterial*);
void SCE_Material_DisableShaderLighting (SCE_SMaterial*);

/* retourne des donnees du materiau */
SCE_STexture* SCE_Material_GetTexture (SCE_SMaterial*, unsigned int);
SCE_SShader* SCE_Material_GetShader (SCE_SMaterial*);

/* ajoute une texture a un materiau */
int SCE_Material_AddTexture (SCE_SMaterial*, SCEuint,
                             SCE_STexture*, int);

/* assigne un shader a un materiau */
void SCE_Material_SetShader (SCE_SMaterial*, SCE_SShader*, int);


/* charge une texture pour le materiau */
int SCE_Material_LoadTextureArg (SCE_SMaterial*, SCEenum,
                                 SCEuint, int, va_list);
int SCE_Material_LoadTexture (SCE_SMaterial*, SCEenum,
                              SCEuint, int, ...);

/* charge un shader pour le materiau */
int SCE_Material_LoadShader (SCE_SMaterial*, const char*, const char*);

#if 0
/* supprime une texture */
void SCE_Material_DeleteTexture (SCE_SMaterial*, int, int);
/* supprime le shader */
void SCE_Material_DeleteShader (SCE_SMaterial*);

/* active/desactive une texture */
void SCE_Material_ActivateTexture (SCE_SMaterial*, int, int, int);
/* active/desactive le shader */
void SCE_Material_ActivateShader (SCE_SMaterial*, int);
#endif

/* defini le materiau actif */
void SCE_Material_Use (SCE_SMaterial*);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
