/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 17/04/2008
   derniere modification : 17/04/2008 */

#include <SCE/SCEMinimal.h>

#include <SCE/interface/SCEBillboard.h>


void SCE_Billboard_Init (SCE_SBillboard *bil)
{
    SCE_Vector3_Set (bil->normal, 0., 0., 0.);
    bil->x = bil->y = bil->z = SCE_TRUE;
    SCE_Vector3_Set (bil->campos, 0., 0., 1.);
}

SCE_SBillboard* SCE_Billboard_Create (void)
{
    SCE_SBillboard *bil = SCE_malloc (sizeof *bil);
    if (!bil)
        Logger_LogSrc ();
    else
        SCE_Billboard_Init (bil);
    return bil;
}
void SCE_Billboard_Delete (SCE_SBillboard *bil)
{
    SCE_free (bil);
}

void SCE_Billboard_SetNormal (SCE_SBillboard *bil, float x, float y, float z)
{
    SCE_Vector3_Set (bil->normal, x, y, z);
    SCE_Vector3_Normalize (bil->normal);
}
void SCE_Billboard_SetNormalv (SCE_SBillboard *bil, SCE_TVector3 v)
{
    SCE_Billboard_SetNormal (bil, v[0], v[1], v[2]);
}

void SCE_Billboard_SetAxis (SCE_SBillboard *bil, int x, int y, int z)
{
    bil->x = x; bil->y = y; bil->z = z;
}
void SCE_Billboard_GetAxis (SCE_SBillboard *bil, int *x, int *y, int *z)
{
    *x = bil->x; *y = bil->y; *z = bil->z;
}

void SCE_Billboard_SetCameraPosition (SCE_SBillboard *bil, float x,
                                      float y, float z)
{
    SCE_Vector3_Set (bil->campos, x, y, z);
}
void SCE_Billboard_SetCameraPositionv (SCE_SBillboard *bil, SCE_TVector3 pos)
{
    SCE_Vector3_Copy (bil->campos, pos);
}

void SCE_Billboard_GetCameraPositionv (SCE_SBillboard *bil, SCE_TVector3 pos)
{
    SCE_Vector3_Copy (pos, bil->campos);
}


void SCE_Billboard_GetMatrix (SCE_SBillboard *bil, SCE_TMatrix4 mat)
{
    float angle;
    SCE_TVector3 crossed, camnor;
    SCE_Vector3_Operator1v (camnor, = -, bil->campos);
    SCE_Vector3_Normalize (camnor);
    angle = asin (SCE_Vector3_Dot (camnor, bil->normal));
    SCE_Vector3_Cross (crossed, bil->normal, camnor);
    SCE_Vector3_Set (crossed, crossed[0]*bil->x,
                     crossed[1]*bil->y, crossed[2]*bil->z);
    SCE_Vector3_Normalize (crossed);
    SCE_Matrix4_Rotatev (mat, angle, crossed);
}
